#ifndef BUCKETS_NODE_H
#include <vector>
#include "SearchNode.h"
using namespace std;

/**
 * BucketsNode.h - Search node for buckets puzzle.  Problem: Given
 * a 5 unit and a 3 unit bucket, how can one measure precisely 4
 * units?  Possible operations: fill/empty 1st/2nd bucket or pour
 * contents of one to the other until source empty or recipient is
 * full.
 *
 * @author Todd Neller
 * @version 1.0
 *

Copyright (C) 2003 Todd Neller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Information about the GNU General Public License is available online at:
  http://www.gnu.org/licenses/
To receive a copy of the GNU General Public License, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */

class BucketsNode : public SearchNode {

 public:

    /**
     * variable <code>bucket1</code> - amount in first bucket */
    int bucket1;


    /**
     * variable <code>bucket2</code> - amount in second bucket */
    int bucket2;


    /**
     * variable <code>MAX_AMOUNT1</code> - max amount in first bucket */
    static const int MAX_AMOUNT1 = 5;


    /**
     * variable <code>MAX_AMOUNT2</code> - max amount in second bucket */
    static const int MAX_AMOUNT2 = 3;

    
    /**
     * Creates a <code>BucketsNode</code> instance and sets it
     * to an initial search state. */
    BucketsNode();


    /**
     * <code>isGoal</code> - test whether or not the current node is a
     * goal node.
     *
     * @return a <code>boolean</code> value */
    bool isGoal();
    

    /**
     * <code>expand</code> - return a (possibly empty) vector of
     * pointers to this node's children
     *
     * @return a pointer to a <code>vector</code> of
     * <code>SearchNode</code> pointers */
    vector<SearchNode *> *expand();
    

    /**
     * <code>deepCopy</code> - return a deep copy of this node.
     *
     * @return an <code>SearchNode *</code> value
     */
    SearchNode *deepCopy();

    ostream& toStream(ostream &sout);
    
};// BucketsNode


/**
 * == - Equality overloading is especially useful for repeated state
 * detection.
 *
 * @param n1 a <code>BucketsNode</code> value - first node
 * @param n2 a <code>BucketsNode</code> value - second node
 * @return a <code>bool</code> value - whether or not the two nodes are equal
 */
bool operator==(BucketsNode n1, BucketsNode n2);

#define BUCKETS_NODE_H
#endif
