#ifndef DEPTH_LIMITED_SEARCHER_H
#include <iostream>
#include <string>
#include "Searcher.h"
using namespace std;

/**
 * DepthLimitedSearcher.h - a simple recursive implementation of
 * depth-limited search.
 *
 * @author Todd Neller
 * @version 1.0
 *

Copyright (C) 2003 Todd Neller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Information about the GNU General Public License is available online at:
  http://www.gnu.org/licenses/
To receive a copy of the GNU General Public License, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */

class DepthLimitedSearcher : public Searcher {

 public:
  
  /** Constructor - set depth limit for search */
  DepthLimitedSearcher(int depthLimit);


  /**
   * <code>search</code> - given an initial node, perform
   * depth-limited search.  This particular implementation is
   * recursive.
   *
   * @param rootNode a <code>SearchNode *</code> value - the initial node
   * @return a <code>bool</code> value - whether or not goal node was
   * found */
  bool search(SearchNode *rootNode);
  

 private:
  
  /**
   * <code>searchRecursive</code> - auxiliary recursive depth-limited
   * search function.
   *
   * @param node a <code>SearchNode *</code> value - the current search node
   * @return a <code>bool</code> value - whether or not goal node
   * was found */
  bool searchRecursive(SearchNode *node);

  /**
   * <code>depthLimit</code> - depth limit of search.  Nodes at the
   * depth limit are treated as if they have no children. */
  int depthLimit;

}; // DepthFirstSearcher

#define DEPTH_LIMITED_SEARCHER_H
#endif
