#ifndef ITERATIVE_DEEPENING_DEPTH_FIRST_SEARCHER_H
#include <iostream>
#include <string>
#include "Searcher.h"
#include "DepthLimitedSearcher.h"
using namespace std;


/**
 * IterativeDeepeningDepthFirstSearcher.h - a simple implementation
 * of iterative-deepening depth-first search.
 *
 * @author Todd Neller
 * @version 1.0
 *

Copyright (C) 2003 Todd Neller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Information about the GNU General Public License is available online at:
  http://www.gnu.org/licenses/
To receive a copy of the GNU General Public License, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */

class IterativeDeepeningDepthFirstSearcher : public Searcher {

 public:
  
  /**
   * Constructor - initialize searcher
   */
  IterativeDeepeningDepthFirstSearcher();


  /**
   * Destructor - free up depth-limited searcher if applicable
   */
  ~IterativeDeepeningDepthFirstSearcher();


  /**
   * <code>search</code> - perform iterative-deepening depth-first
   * search by performing successive depth-limited searches with
   * increasing depth-limits.  This particular implementation of
   * ID-DFS is recursive.
   *
   * @param rootNode a <code>SearchNode *</code> value - the initial node
   * @return a <code>bool</code> value - whether or not goal node was
   * found */
  bool search(SearchNode *rootNode);


 private:
  /* <code>searcher</code> - depth-limited searcher for each search
     iteration. Null if no search performed.  Otherwise, holds last
     successful depth-limited searcher. */
  DepthLimitedSearcher *searcher;

}; // IterativeDeepeningDepthFirstSearcher

#define ITERATIVE_DEEPENING_DEPTH_FIRST_SEARCHER_H
#endif
