#include "SearchNode.h"

/**
 * SearchNode.cc - a simple node for uninformed AI search (assuming
 * cost equals depth).
 *
 * @author Todd Neller
 * @version 1.0
 *

Copyright (C) 2003 Todd Neller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Information about the GNU General Public License is available online at:
  http://www.gnu.org/licenses/
To receive a copy of the GNU General Public License, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */


/**
 * constructor - extend if your search node needs extra initialization
 */
SearchNode::SearchNode() {
  parent = 0;
  depth = 0;
}


/**
 * default destructor - override if your search node dynamically
 * allocates memory.
 */
SearchNode::~SearchNode(){
}

  

/**
 * <code>childClone</code> - returns a clone of this node that has
 * been made a child of this node and has a depth one greater than
 * this.
 *
 * @return a <code>SearchNode</code> value */
SearchNode *SearchNode::childClone() {
  SearchNode *child = this->deepCopy();
  child->parent = this;
  child->depth = depth + 1;
  return child;
}


/**
 * <code>toStream</code> - print a text representation of this search
 * node to the given output stream.  You are encouraged to override
 * this method for subclasses in order to provide more node-specific
 * information. */
ostream& SearchNode::toStream(ostream &sout) 
{
  sout << "SearchNode[addr=" << hex << this << ", depth=" << dec << depth 
       << ", parent=" << hex <<  parent << dec << "]";
  return sout;
}


/**
 * Overloading << is especially useful for debugging, tracing search
 * execution, reporting results, etc.  This default implementation is
 * not particularly informative, so it's recommended that you overload
 * << for each type of SearchNode you create.
 *
 * @param sout an <code>ostream&</code> value - output stream
 * @param n a <code>SearchNode</code> value - reference to
 * node to be printed to output stream
 * @return an <code>ostream&</code> value - same output stream 
 */
ostream& operator<<(ostream &sout, SearchNode &n) 
{
  return n.toStream(sout);
}

ostream& operator<<(ostream &sout, SearchNode *n) 
{
  return n->toStream(sout);
}
