#include "Searcher.h"

/**
 * Searcher.cc - a superclass for AI searcher classes.
 *
 * @author Todd Neller
 * @version 1.0
 *

Copyright (C) 2003 Todd Neller

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

Information about the GNU General Public License is available online at:
  http://www.gnu.org/licenses/
To receive a copy of the GNU General Public License, write to the Free
Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.

 */


/**
 * Default constructor - initialize nodeCount and goalNode to 0
 */
Searcher::Searcher() {
  nodeCount = 0;
  goalNode = 0;
}


/**
 * Destructor - free up goal node and ancestors if applicable
 */
Searcher::~Searcher() {
  while (goalNode != 0) {
    SearchNode *temp = goalNode->parent;
    delete goalNode;
    goalNode = temp;
  }
}


/**
 * <code>getGoalNode</code> - Returns a goal node if the previous
 * search was successful, and null otherwise.
 *
 * @return a <code>SearchNode *</code> value - the goal node from
 * previous search or null if no goal node was found */
SearchNode *Searcher::getGoalNode() {
  return goalNode;
}
  
    
/**
 * <code>getNodeCount</code> - Returns the number of nodes
 * examined (goal-checked) in the previous search.
 *
 * @return an <code>int</code> value - the number of nodes checked
 * in the previous search.  This may be considerably less than the
 * number of children generated. */
int Searcher::getNodeCount() {
  return nodeCount;
}
  
  
/**
 * <code>printGoalPath</code> - If the previous search was
 * successful, print each node along the goal path in sequence
 * starting with the root node.  Otherwise, print "Goal node not
 * found".  */
void Searcher::printGoalPath() {
  if (goalNode == 0)
    cout << "Goal node not found" << endl;
  else
    printPath(*goalNode);
}

  
/**
 * <code>printPath</code> - print each node along the path to the
 * given node in sequence starting with the root node.
 *
 * @param node a <code>SearchNode</code> value */
void Searcher::printPath(SearchNode &node) {
  if (node.parent != 0)
    printPath(*(node.parent));
  cout << node << endl;
}  


